from django.db.models.signals import post_save
from django.dispatch import receiver
from django_msteams_notify.models import TeamsNotification
from core.utils import send_teams_message

@receiver(post_save, sender=TeamsNotification)
def send_notification_signal(sender, instance, created, **kwargs):
    if created:
        send_teams_message(instance)



from django.urls import path
from django_msteams_notify.views import (
    TeamsNotificationListAPIView,
    TeamsNotificationCreateAPIView,
    TeamsNotificationDetailAPIView
)

urlpatterns = [
    path('send-teams/', TeamsNotificationListAPIView.as_view(), name='teamsnotifications-list'),
    path('send-teams/create/', TeamsNotificationCreateAPIView.as_view(), name='teamsnotifications-create'),
    path('send-teams/<uuid:uuid>/', TeamsNotificationDetailAPIView.as_view(), name='teamsnotifications-detail'),
]



from rest_framework import serializers
from django_msteams_notify.models import TeamsNotification

class TeamsNotificationSerializer(serializers.ModelSerializer):
    class Meta:
        model = TeamsNotification
        fields = ['id', 'message', 'status', 'sent_at']



from django.contrib import admin
from django_msteams_notify.models import TeamsNotification

@admin.register(TeamsNotification)
class TeamsNotificationAdmin(admin.ModelAdmin):
    list_display = ("message", "status", "sent_at")
    list_filter = ("status", "sent_at")
    search_fields = ("message",)



from django.db import models

class TeamsNotification(models.Model):
    STATUS_CHOICES = [
        ("pending", "Pending"),
        ("sent", "Sent"),
        ("failed", "Failed"),
    ]
    message = models.TextField()
    sent_at = models.DateTimeField(auto_now_add=True)
    status = models.CharField(max_length=20, choices=STATUS_CHOICES, default="pending")  

    def __str__(self):
        return f"{self.sent_at} - {self.status}"



from django.apps import AppConfig


class NotifyConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'django_msteams_notify'

    def ready(self):
       import django_msteams_notify.signals 





from rest_framework.generics import ListAPIView, CreateAPIView, RetrieveUpdateDestroyAPIView
from django_msteams_notify.models import TeamsNotification
from django_msteams_notify.serializers import TeamsNotificationSerializer
from django_msteams_notify.utils import send_teams_message

class TeamsNotificationListAPIView(ListAPIView):
    queryset = TeamsNotification.objects.all().order_by('-sent_at')
    serializer_class = TeamsNotificationSerializer

class TeamsNotificationCreateAPIView(CreateAPIView):
    serializer_class = TeamsNotificationSerializer

    def perform_create(self, serializer):
        notification = serializer.save()
        send_teams_message(notification)  

class TeamsNotificationDetailAPIView(RetrieveUpdateDestroyAPIView):
    queryset = TeamsNotification.objects.all()
    serializer_class = TeamsNotificationSerializer



import requests
from django.conf import settings
from django_msteams_notify.models import TeamsNotification

def send_teams_message(notification: TeamsNotification):
    """Send a Teams message and update notification status."""
    webhook_url = getattr(settings, "TEAMS_WEBHOOK_URL", None)

    if not webhook_url:
        notification.status = "failed"
        notification.save(update_fields=["status"])
        return False

    try:
        payload = {"text": notification.message}
        response = requests.post(webhook_url, json=payload)

        if response.status_code == 200:
            notification.status = "sent"
        else:
            notification.status = "failed"

    except Exception:
        notification.status = "failed"

    notification.save(update_fields=["status"])
    return notification.status == "sent"
